------------------------------------------------------------------------------
--								Keybindings support							--
------------------------------------------------------------------------------
local Keybindings =  FlexBar2:NewModule("Keybindings");
-- If this var is incremented, SetDefaults() will be called for this module thus resetting all its settings.
Keybindings.Version = 1;
local BindingCache = {};

-- Steal Ace2's validity check function for our own use
function Keybindings:ValidateBinding(Key)
    return AceLibrary("AceConsole-2.0").keybindingValidateFunc(Key);
end

-- Declare the button mixin
Keybindings.ButtonMixin = FlexBar2.OO.Mixin
{
	"SetBinding",	
};

-- Load currently set binding
function Keybindings.ButtonMixin:LoadSettings()
    self:SetBinding(self.InfoTable.KeyBinding);
end

-- Set a binding
function Keybindings.ButtonMixin:SetBinding(Key)
	-- Get the 'old' binding if any
    local SavedBinding = self.InfoTable.KeyBinding;
    FlexBar2:Debug("Setting binding to [" , Key,  "] after verif [", Keybindings:ValidateBinding(Key) ,"] from old binding [", SavedBinding, "]");
	-- Clear any bindings currently on the frame before setting the new one
    ClearOverrideBindings(self.Frame);
	-- Make sure the binding is valid before setting it
    if(Keybindings:ValidateBinding(Key)) then
        FlexBar2:Debug("Verif passed");
		-- Set it !
        SetOverrideBindingClick(self.Frame, nil, Key, self.Frame:GetName());
    end
    if(type(SavedBinding) == "string") then -- Clear the old binding from the cache
        BindingCache[SavedBinding] = nil;
    end
	-- Clear the new binding from any old buttons it might have been set to
    if(type(Key) == "string") then
        if(type(BindingCache[Key]) == "table") then
            BindingCache[Key]:SetBinding();
        end
        BindingCache[Key] = self;
    end
	-- Save our newly bound key
    self.InfoTable.KeyBinding = Key;
end

-- GUI --
local Waterfall = FlexBar2.Waterfall;
local KeyBinding;
function Keybindings.ButtonWaterfall(Action, ButtonName, ButtonTable)
	if(Action == "edit") then
		if(#ButtonTable == 1) then
			-- Keybinding setter --
            KeyBinding = FlexBar2:GetSetting(ButtonTable, "KeyBinding");
            Waterfall:AddControl(
            "type", "label",
            "text", "Keybinding: ");
            Waterfall:AddControl(
            "type", "keybind",
            "width", "50",
            "noNewLine", true,
            "causeRefresh", false,
            "setFunc", function(NewBinding) KeyBinding = NewBinding; end,
            "getFunc", function() return KeyBinding; end);
        end
	elseif(Action == "save") then
		-- Save our new binding and set it
        FlexBar2:Debug("KeyBinding: ", KeyBinding);
        local ButtonList = FlexBar2.Buttons;
        ButtonList[ButtonTable]:SetBinding(KeyBinding);
    end
end

-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;

-- /fb setbinding command
local function SlashCommand_SetBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    if(#ButtonTable ~= 1) then FlexBar2:Print("Cannot set the same keybinding on multiple buttons"); return; end
    local ButtonList = FlexBar2.Buttons;
    for _, ButtonName in ipairs(ButtonTable) do
       ButtonList[ButtonName]:SetBinding(Params.binding);
    end
end

-- /fb clearbinding command
local function SlashCommand_ClearBinding(Params)
    local ButtonTable = FlexBar2:ToButtonTable(Params.button);
    local ButtonList = FlexBar2.Buttons;
    for _, ButtonName in ipairs(ButtonTable) do
        ButtonList[ButtonName]:SetBinding();
    end
end

-- Localize this neat verify function for faster access
local VerifyExistance = FlexBar2.SlashCommands.VerifyExistance;

-- Helper verification function
local VerifyBinding = function(Binding)
    if(Keybindings:ValidateBinding(Binding)) then
        return true;
    else
        return "Invalid keybinding";
    end
end

-- AddCommand format :AddCommand(Command, Function, Group, Help)
-- AddParam format :AddParam(Command, Param, Required, AllowedValues, DefaultValue, VerifyFunc) 
-- everything here should be self explainative knowing the syntax
SlashCommands:AddCommand("setbinding", SlashCommand_SetBinding, "keybinding", "/FlexBar2 setbinding button=<Button> binding=<Key>");
	SlashCommands:AddParam("setbinding", "button", true, nil, nil, VerifyExistance);
	SlashCommands:AddParam("setbinding", "binding", true, nil, nil, VerifyBinding);

SlashCommands:AddCommand("clearbinding", SlashCommand_ClearBinding, "keybinding", "/FlexBar2 clearbinding button=<Button>");
	SlashCommands:AddParam("clearbinding", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end);
